/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.io.*;

import com.dickimawbooks.texparserlib.*;
import com.dickimawbooks.texparserlib.latex.*;

public class BookTeXApp implements TeXApp
{
   public BookTeXApp(MakeJmlrBookGUI app)
   {
      this.app = app;

      errBuilder = new StringBuilder();
      warnBuilder = new StringBuilder();
   }

   public String kpsewhich(String arg)
     throws IOException,InterruptedException
   {
      return app.kpsewhich(arg);
   }

   public void epstopdf(File epsFile, File pdfFile)
     throws IOException,InterruptedException
   {
      app.epstopdf(epsFile, pdfFile);
   }

   public void wmftoeps(File wmfFile, File epsFile)
     throws IOException,InterruptedException
   {
      app.wmftoeps(wmfFile, epsFile);
   }

   public void copyFile(File orgFile, File newFile)
     throws IOException,InterruptedException
   {
      app.copyFile(orgFile, newFile);
   }

   public boolean isReadAccessAllowed(TeXPath path)
   {
      return isReadAccessAllowed(path.getFile());
   }

   public boolean isReadAccessAllowed(File file)
   {
      return file.canRead();
   }

   public boolean isWriteAccessAllowed(TeXPath path)
   {
      return isWriteAccessAllowed(path.getFile());
   }

   public boolean isWriteAccessAllowed(File file)
   {
      return file.canWrite();
   }

   public void progress(int progress)
   {
   }

   // TeXApp method
   public void warning(TeXParser parser, String msg)
   {
      warn(parser.getLineNumber(), msg);
   }

   public void warn(int lineNum, String msg)
   {
      if (warnBuilder.length() > 0)
      {
         warnBuilder.append("\n");
      }

      if (lineNum > -1)
      {
         warnBuilder.append(
           MakeJmlrBookGUI.getLabelWithValues("error.line",
               ""+lineNum, msg));
      }
      else
      {
         warnBuilder.append(msg);
      }
   }

   public void substituting(TeXParser parser, String original, String replacement)
   {
      if (warnBuilder.length() > 0)
      {
         warnBuilder.append(String.format("%n"));
      }

      String message;

      if (replacement.isEmpty())
      {
         message =
            MakeJmlrBookGUI.getLabelWithValue("warning.removing",
             original);
      }
      else
      {
         message = MakeJmlrBookGUI.getLabelWithValues("warning.substituting",
              original, replacement);

      }

      int lineNum = parser.getLineNumber();
      File file = parser.getCurrentFile();

      if (lineNum > 0)
      {
         message = MakeJmlrBookGUI.getLabelWithValues("error.line",
           ""+lineNum, message);
      }

      if (file != null)
      {
         message = MakeJmlrBookGUI.getLabelWithValues("error.file",
           file.getName(), message);
      }

      warnBuilder.append(message);
   }

   public void message(String msg)
   {
      app.message(msg);
   }

   public void error(Exception e)
   {
      if (errBuilder.length() > 0)
      {
         errBuilder.append("\n");
      }

      if (e instanceof TeXSyntaxException)
      {
         errBuilder.append(((TeXSyntaxException)e).getMessage(this));
      }
      else
      {
         errBuilder.append(e.getMessage());
      }
   }

   public String getErrors()
   {
      return errBuilder.toString();
   }

   public String getWarnings()
   {
      return warnBuilder.toString();
   }

   public String getMessage(String label)
   {
      return app.getLabel(label);
   }

   public String getMessage(String label, String param)
   {
      return app.getLabelWithValue(label, param);
   }

   public String getMessage(String label, String[] params)
   {
      return app.getLabelWithValues(label, params);
   }

   public String getMessage(String label, Object... params)
   {
      return app.getLabelWithValues(label, params);
   }

   public String requestUserInput(String message)
     throws IOException
   {
      return javax.swing.JOptionPane.showInputDialog(app, message);
   }

   private MakeJmlrBookGUI app;

   private StringBuilder errBuilder, warnBuilder;
}
