/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui;

import java.net.*;
import java.nio.file.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;

public class InformationDialog extends JDialog implements ActionListener
{
   public InformationDialog(MakeJmlrBookGUI app)
   {
      super(app, "");
      this.application = app;

      infoPane = new JEditorPane();
      infoPane.setEditable(false);
      infoPane.setContentType("text/html");

      HTMLEditorKit kit = new HTMLEditorKit();
      infoPane.setEditorKit(kit);

      StyleSheet styleSheet = kit.getStyleSheet();
      styleSheet.addRule("dt {font-weight: bold;}");

      Document doc = kit.createDefaultDocument();
      infoPane.setDocument(doc);

      scrollPane = new JScrollPane(infoPane);
      getContentPane().add(scrollPane, "Center");

      JPanel buttonPanel = new JPanel();
      getContentPane().add(buttonPanel, "South");

      buttonPanel.add(JMLRResources.createCloseButton(this));

      Toolkit tk = Toolkit.getDefaultToolkit();
      Dimension dim = tk.getScreenSize();
      dim.width = (int)(dim.getWidth()*0.5);
      dim.height = (int)(dim.getHeight()*0.5);
      scrollPane.setPreferredSize(dim);

      pack();

      setLocationRelativeTo(application);
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("close"))
      {
         setVisible(false);
      }
   }

   public void display(String label)
   {
      setTitle(MakeJmlrBookGUI.getLabel("info."+label+".title"));
      infoPane.setText(MakeJmlrBookGUI.getLabel("info."+label+".details"));

      setVisible(true);

      showAndScrollToTop();
   }

   public void display(String label, String[] params)
   {
      setTitle(MakeJmlrBookGUI.getLabel("info."+label+".title"));

      infoPane.setText(MakeJmlrBookGUI.getLabelWithValues("info."+label+".details", params));

      showAndScrollToTop();
   }

   public void showAndScrollToTop()
   {
      setVisible(true);

      SwingUtilities.invokeLater(new Runnable()
      {
         public void run()
         {
             infoPane.scrollRectToVisible(new Rectangle(1,1,1,1));
         }
      });
   }

   private JEditorPane infoPane;

   private JScrollPane scrollPane;

   private MakeJmlrBookGUI application;
}
