/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Vector;

import javax.swing.*;

import com.dickimawbooks.texparserlib.*;
import com.dickimawbooks.texparserlib.bib.*;

import com.dickimawbooks.makejmlrbookgui.*;

public class ArticlesPanel extends JPanel
  implements ActionListener,NewBookPanel
{
   public ArticlesPanel(BookPartPanel bookPartPanel, NewBookDialog dialog)
   {
      super(null);
      this.bookPartPanel = bookPartPanel;
      this.dialog = dialog;

      setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
      setAlignmentX(LEFT_ALIGNMENT);

      JLabel label;
      Box box;

      cardLayout = new CardLayout();
      topComp = new JPanel(cardLayout);
      topComp.setAlignmentX(LEFT_ALIGNMENT);
      add(topComp);

      partVector = new Vector<PartListPanel>();

      partComp = new JPanel(new BorderLayout());
      partComp.setAlignmentX(LEFT_ALIGNMENT);
      cardLayout.addLayoutComponent(partComp, "part");
      topComp.add(partComp);

      partListsComp = Box.createVerticalBox();
      partListsComp.setAlignmentX(LEFT_ALIGNMENT);
      partComp.add(partListsComp, BorderLayout.CENTER);

      JButton addButton = JMLRResources.createActionButton("newbook.articles",
       "addpart", this, null);
      JPanel buttonPanel = new JPanel();
      buttonPanel.add(addButton);
      partComp.add(buttonPanel, BorderLayout.SOUTH);

      addPart();

      mainComp = Box.createVerticalBox();
      mainComp.setAlignmentX(LEFT_ALIGNMENT);
      mainComp.setBorder(BorderFactory.createLoweredSoftBevelBorder());
      cardLayout.addLayoutComponent(mainComp, "main");
      topComp.add(mainComp);

      label = new JLabel(MakeJmlrBookGUI.getLabel("newbook.articles.main"));
      label.setAlignmentX(LEFT_ALIGNMENT);
      mainComp.add(label);

      mainImpPapersPanel = new ImportPapersPanel(dialog, this, MAIN);
      mainComp.add(mainImpPapersPanel, BorderLayout.CENTER);

      apdComp = new JPanel(new BorderLayout());
      apdComp.setAlignmentX(LEFT_ALIGNMENT);
      apdComp.setBorder(BorderFactory.createLoweredSoftBevelBorder());
      add(apdComp);

      label = new JLabel(MakeJmlrBookGUI.getLabel("newbook.articles.appendix"));
      label.setAlignmentX(LEFT_ALIGNMENT);
      apdComp.add(label, BorderLayout.NORTH);

      apdImpPapersPanel = new ImportPapersPanel(dialog, this, APPENDIX);
      apdComp.add(apdImpPapersPanel, BorderLayout.CENTER);
   }

   public synchronized PartListPanel addPart()
   {
      if (partVector.size() > 0)
      {
         partVector.lastElement().setMovePartDownEnabled(true);
      }

      PartListPanel partPanel = new PartListPanel(this, partVector.size(),
         dialog);
      partPanel.setBorder(BorderFactory.createLoweredSoftBevelBorder());

      partVector.add(partPanel);
      partListsComp.add(partPanel);

      partPanel = partVector.firstElement();

      if (partVector.size() == 1)
      {
         partPanel.setMovePartUpEnabled(false);
         partPanel.setRemoveEnabled(false);
      }
      else
      {
         partPanel.setRemoveEnabled(true);
      }

      return partPanel;
   }

   public void removePart(int index)
   {
      partListsComp.remove(index);
      partVector.remove(index);

      for (int i = index, n = partVector.size(); i < n; i++)
      {
         PartListPanel partPanel = partVector.get(i);
         partPanel.setIndex(i);
      }

      if (partVector.size() == 1)
      {
         partVector.firstElement().setRemoveEnabled(false);
      }

      partVector.firstElement().setMovePartUpEnabled(false);
      partVector.lastElement().setMovePartDownEnabled(false);

      revalidate();
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("addpart"))
      {
         addPart();
         revalidate();
      }
   }

   public boolean hasNextButton() {return true;}

   public boolean hasBackButton() {return true;}

   public void updatePanel()
   {
      if (bookPartPanel.getPartConfig() == BookPartPanel.PARTS_NO)
      {
         cardLayout.show(topComp, "main");

         mainImpPapersPanel.updatePanel();
      }
      else
      {
         cardLayout.show(topComp, "part");

         for (PartListPanel panel : partVector)
         {
            panel.updatePanel();
         }
      }

      apdComp.setVisible(bookPartPanel.hasAppendices());

      if (apdComp.isVisible())
      {
         apdImpPapersPanel.updatePanel();
      }
   }

   public void removeEmptyDefaultParts()
   {
      String defTitle = MakeJmlrBookGUI.getLabel("newbook.articles.partdefault");

      for (int i = partVector.size()-1; i >= 0; i--)
      {
         PartListPanel panel = partVector.get(i);

         if (panel.countPapers() == 0
          && panel.getPartTitle().equals(defTitle))
         {
            removePart(i);
         }
      }
   }

   public PartListPanel getPart(int index)
   {
      if (index < 0 || index >= partVector.size())
      {
         return null;
      }

      return partVector.get(index);
   }

   public Vector<PartListPanel> getPartLists()
   {
      return partVector;
   }

   public ImportPapersPanel getMainImportPapers()
   {
      return mainImpPapersPanel;
   }

   public ImportPapersPanel getAppendixImportPapers()
   {
      return apdImpPapersPanel;
   }

   public boolean canMoveForwards()
   {
      int total = 0;

      if (bookPartPanel.getPartConfig() == BookPartPanel.PARTS_NO)
      {
         total = mainImpPapersPanel.countPapers();
      }
      else
      {
         for (PartListPanel panel : partVector)
         {
            total += panel.countPapers();
         }
      }

      if (total > 0)
      {
         return true;
      }

      JMLRResources.error(this,
          MakeJmlrBookGUI.getLabel("error.newbook.no_main_papers"));

      return false;
   }

   public void addCimlImportFile(File base, String section, BibEntry entry,
     TeXParser parser)
   {
       mainImpPapersPanel.addCimlImportFile(base, entry, parser);

       if (section == null)
       {
          partVector.firstElement().addCimlImportFile(base, entry, parser);
          return;
       }

       for (PartListPanel panel : partVector)
       {
          if (panel.getPartTitle().equals(section))
          {
             panel.addCimlImportFile(base, entry, parser);
             return;
          }
       }

       PartListPanel panel = addPart();
       panel.setPartTitle(section);

       panel.addCimlImportFile(base, entry, parser);
   }

   public void clearImportData()
   {
      partVector.clear();
      partListsComp.removeAll();

      addPart();

      mainImpPapersPanel.removeAll();
      mainImpPapersPanel.addRow();
   }

   public ImportPapersPanel getImportPapersPanel(ImportPaperPanel panel)
   {
      int idx = mainImpPapersPanel.getImportPaperIndex(panel);

      if (idx != -1)
      {
         return mainImpPapersPanel;
      }

      idx = apdImpPapersPanel.getImportPaperIndex(panel);

      if (idx != -1)
      {
         return apdImpPapersPanel;
      }

      for (PartListPanel part : partVector)
      {
         ImportPapersPanel impPanel = part.getImportPapersPanel();

         idx = impPanel.getImportPaperIndex(panel);

         if (idx != -1)
         {
            return impPanel;
         }
      }

      return null;
   }

   public ImportPapersPanel getImportPapersPanel(File file)
   {
      int idx = mainImpPapersPanel.getImportPaperIndex(file);

      if (idx != -1)
      {
         return mainImpPapersPanel;
      }

      idx = apdImpPapersPanel.getImportPaperIndex(file);

      if (idx != -1)
      {
         return apdImpPapersPanel;
      }

      for (PartListPanel part : partVector)
      {
         ImportPapersPanel impPanel = part.getImportPapersPanel();

         idx = impPanel.getImportPaperIndex(file);

         if (idx != -1)
         {
            return impPanel;
         }
      }

      return null;
   }

   public ImportPaperPanel getTransferComponent(String data)
   {
      String[] split = data.split(":");

      if (split.length != 2)
      {
         return null;
      }

      String parentName = split[0];

      int index;

      try
      {
         index = Integer.parseInt(split[1]);
      }
      catch (NumberFormatException e)
      {
         return null;
      }

      if (parentName.equals(MAIN))
      {
         return (ImportPaperPanel)mainImpPapersPanel.getComponent(index);
      }

      if (parentName.equals(APPENDIX))
      {
         return (ImportPaperPanel)apdImpPapersPanel.getComponent(index);
      }

      for (PartListPanel part : partVector)
      {
         ImportPapersPanel impPanel = part.getImportPapersPanel();

         if (parentName.equals(impPanel.getName()))
         {
            return (ImportPaperPanel)impPanel.getComponent(index);
         }
      }

      return null;
   }

   private BookPartPanel bookPartPanel;

   private JComponent partComp, mainComp, apdComp, topComp,
     partListsComp;

   private ImportPapersPanel mainImpPapersPanel, apdImpPapersPanel;

   private Vector<PartListPanel> partVector;

   private CardLayout cardLayout;

   private NewBookDialog dialog;

   public static final String MAIN="main";
   public static final String APPENDIX="appendix";
   public static final String PART="part";
}
