/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;

import com.dickimawbooks.makejmlrbookgui.*;

public class BookDirectoryPanel extends JPanel
 implements NewBookPanel
{
   public BookDirectoryPanel(NewBookDialog dialog)
   {
      super(null);
      this.dialog = dialog;
      setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

      Box box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);
      add(box);

      nameField = new JTextField(12);

      int volNum = dialog.getVolume();

      dirField = new FileField(this,
         MakeJmlrBookGUI.getLabelWithValue("newbook.default_dir", 
           volNum == 0 ? "" : ""+volNum),
         dialog.getDirChooser(),
         JFileChooser.DIRECTORIES_ONLY);

      JLabel label = new JLabel(MakeJmlrBookGUI.getLabel("newbook.dir"));
      label.setDisplayedMnemonic(MakeJmlrBookGUI.getMnemonic("newbook.dir"));
      label.setLabelFor(nameField);

      Dimension dim = nameField.getPreferredSize();
      dim.width = Integer.MAX_VALUE;
      nameField.setMaximumSize(dim);

      box.add(label);
      box.add(nameField);

      label = new JLabel(MakeJmlrBookGUI.getLabel("newbook.dir_in"));
      label.setDisplayedMnemonic(MakeJmlrBookGUI.getMnemonic("newbook.dir_in"));
      label.setLabelFor(dirField.getTextField());

      box.add(label);
      box.add(dirField);

      add(Box.createVerticalStrut(10));

      box = Box.createHorizontalBox();
      box.setAlignmentX(Component.LEFT_ALIGNMENT);
      add(box);

      bookFontsArea = new JTextArea(String.format("%s%n%s%n%s%n%s%n%n",
       "\\usepackage{mathptmx}",
       "\\usepackage[scaled=0.9]{helvet}",
       "\\usepackage{courier}",
       "\\usepackage{cmap}"));

      label = new JLabel(MakeJmlrBookGUI.getLabel("newbook.fonts"));
      label.setDisplayedMnemonic(MakeJmlrBookGUI.getMnemonic("newbook.fonts"));
      label.setLabelFor(bookFontsArea);

      dim = bookFontsArea.getPreferredSize();
      dim.width = Integer.MAX_VALUE;

      JScrollPane sp = new JScrollPane(bookFontsArea);
      sp.setMaximumSize(dim);

      label.setAlignmentX(Component.LEFT_ALIGNMENT);
      label.setAlignmentY(Component.TOP_ALIGNMENT);
      sp.setAlignmentY(Component.TOP_ALIGNMENT);

      box.add(label);
      box.add(sp);
      box.setMaximumSize(dim);

   }

   public boolean canMoveForwards() 
   {
      if (nameField.getText().isEmpty())
      {
         JMLRResources.error(this,
            MakeJmlrBookGUI.getLabel("newbook.missing_base"));
         return false;
      }

      File file = getBookDirectory();

      if (file.exists())
      {
         JMLRResources.error(this, 
            MakeJmlrBookGUI.getLabelWithValue("error.file_exists",
            file.getAbsolutePath()));

         return false;
      }

      if (!file.mkdir())
      {
         JMLRResources.error(this,
           MakeJmlrBookGUI.getLabelWithValue("error.io.cant_mkdir",
              file.getAbsolutePath()));

         return false;
      }

      return true;
   }

   public boolean hasNextButton() {return true;}

   public boolean hasBackButton() {return true;}

   public void updatePanel()
   {
      JFileChooser fc = dialog.getDirChooser();

      int volNum = dialog.getVolume();

      nameField.setText(
         MakeJmlrBookGUI.getLabelWithValue("newbook.default_dir", 
           volNum == 0 ? "" : ""+volNum));

      File file = fc.getCurrentDirectory();

      dirField.setFile(file);
   } 

   public File getBookDirectory()
   {
      return new File(dirField.getFile(), nameField.getText());
   }

   public String getBookFonts()
   {
      return bookFontsArea.getText();
   }

   private NewBookDialog dialog;

   private JTextField nameField;

   private FileField dirField;

   private JTextArea bookFontsArea;
}
