/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.awt.*;

import javax.swing.*;

import com.dickimawbooks.makejmlrbookgui.*;

public class BookPartPanel extends JPanel
  implements NewBookPanel
{
   public BookPartPanel(NewBookDialog dialog)
   {
      super(null);
      this.dialog = dialog;

      setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
      setAlignmentX(LEFT_ALIGNMENT);

      topLabel = new JLabel(MakeJmlrBookGUI.getLabel("newbook.parts.query"));
      topLabel.setAlignmentX(LEFT_ALIGNMENT);
      add(topLabel);

      ButtonGroup bg = new ButtonGroup();

      noPartButton = JMLRResources.createRadioButton(
        "newbook.parts", "partno", null, true);
      bg.add(noPartButton);
      add(noPartButton);
      noPartButton.setAlignmentX(LEFT_ALIGNMENT);

      tocButton = JMLRResources.createRadioButton(
        "newbook.parts", "parttoc", null, false);
      bg.add(tocButton);
      add(tocButton);
      tocButton.setAlignmentX(LEFT_ALIGNMENT);

      yesPartButton = JMLRResources.createRadioButton(
        "newbook.parts", "partyes", null, false);
      bg.add(yesPartButton);
      add(yesPartButton);
      yesPartButton.setAlignmentX(LEFT_ALIGNMENT);

      add(Box.createVerticalStrut(INDENT));
      add(Box.createVerticalGlue());

      apdLabel = new JLabel(MakeJmlrBookGUI.getLabel("newbook.parts.appendices"));
      apdLabel.setAlignmentX(LEFT_ALIGNMENT);
      add(apdLabel);

      cimlText = MakeJmlrBookGUI.getLabel("newbook.parts.apdinfo");
      jmlrText = MakeJmlrBookGUI.getLabel("newbook.parts.jmlr");

      textArea = new JTextArea();
      textArea.setRows(2);
      textArea.setLineWrap(true);
      textArea.setWrapStyleWord(true);
      textArea.setEditable(false);
      textArea.setOpaque(false);
      textArea.setAlignmentX(LEFT_ALIGNMENT);
      textArea.setText(cimlText);
      add(textArea);

      Dimension dim = textArea.getMaximumSize();
      dim.height = (int)textArea.getPreferredSize().getHeight();
      textArea.setMaximumSize(dim);

      bg = new ButtonGroup();

      noApdButton = JMLRResources.createRadioButton(
         "newbook.parts", "apdno", null, true);
      noApdButton.setAlignmentX(LEFT_ALIGNMENT);
      bg.add(noApdButton);
      add(noApdButton);

      yesApdButton = JMLRResources.createRadioButton(
         "newbook.parts", "apdyes", null, false);
      yesApdButton.setAlignmentX(LEFT_ALIGNMENT);
      bg.add(yesApdButton);
      add(yesApdButton);

      add(Box.createVerticalGlue());
      add(Box.createVerticalStrut(200));
   }

   public boolean hasAppendices()
   {
      return yesApdButton.isSelected();
   }

   public int getPartConfig()
   {
      if (noPartButton.isSelected())
      {
         return PARTS_NO;
      }
      else if (tocButton.isSelected())
      {
         return PARTS_TOC;
      }

      return PARTS_YES;
   }

   public boolean hasParts()
   {
      return getPartConfig() != PARTS_NO;
   }

   public void updatePanel()
   {
      if (dialog.isCiML())
      {
         textArea.setText(cimlText);
         noPartButton.setVisible(true);
         yesPartButton.setVisible(true);
         tocButton.setVisible(true);
         noApdButton.setVisible(true);
         yesApdButton.setVisible(true);
         apdLabel.setVisible(true);
         topLabel.setVisible(true);
      }
      else
      {
         textArea.setText(jmlrText);
         noPartButton.setSelected(true);
         noApdButton.setSelected(true);
         noPartButton.setVisible(false);
         yesPartButton.setVisible(false);
         tocButton.setVisible(false);
         noApdButton.setVisible(false);
         yesApdButton.setVisible(false);
         apdLabel.setVisible(false);
         topLabel.setVisible(false);
      }
   }

   public void setParts(boolean hasParts)
   {
      if (hasParts)
      {
         if (yesPartButton.isSelected() || tocButton.isSelected())
         {
            return;
         }

         yesPartButton.setSelected(true);
      }
      else
      {
         noPartButton.setSelected(true);
      }
   }

   public boolean canMoveForwards() {return true;}

   public boolean hasNextButton() {return true;}

   public boolean hasBackButton() {return true;}

   private static final int INDENT = 20;

   private JRadioButton noPartButton, yesPartButton, tocButton,
    noApdButton, yesApdButton;

   public static final int PARTS_NO=0, PARTS_TOC=1, PARTS_YES=2;

   private NewBookDialog dialog;

   private JLabel topLabel, apdLabel;
   private String cimlText, jmlrText;
   private JTextArea textArea;
}
