/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;

import com.dickimawbooks.makejmlrbookgui.*;

public class FrontMatterPanel extends JPanel
  implements ActionListener,NewBookPanel
{
   public FrontMatterPanel(NewBookDialog dialog)
   {
      super(null);

      this.dialog = dialog;

      setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
      setAlignmentX(LEFT_ALIGNMENT);

      JLabel label 
        = new JLabel(MakeJmlrBookGUI.getLabel("newbook.frontmatter.preface"));
      label.setAlignmentX(LEFT_ALIGNMENT);

      add(label);

      ButtonGroup bg = new ButtonGroup();

      prefaceNoButton = 
         JMLRResources.createRadioButton("newbook.frontmatter", "prefaceno",
         this, false);
      prefaceNoButton.setAlignmentX(LEFT_ALIGNMENT);
      bg.add(prefaceNoButton);
      add(prefaceNoButton);

      prefaceYesButton = 
         JMLRResources.createRadioButton("newbook.frontmatter", "prefaceyes",
         this, true);
      prefaceYesButton.setAlignmentX(LEFT_ALIGNMENT);
      bg.add(prefaceYesButton);
      add(prefaceYesButton);

      Box box = Box.createHorizontalBox();
      box.setAlignmentX(LEFT_ALIGNMENT);
      add(box);

      box.add(Box.createHorizontalStrut(INDENT));

      bg = new ButtonGroup();

      prefaceNewButton = JMLRResources.createRadioButton(
         "newbook.frontmatter", "prefacenew", this, true);
      bg.add(prefaceNewButton);
      box.add(prefaceNewButton);
      prefaceNewButton.setAlignmentX(LEFT_ALIGNMENT);

      box = Box.createHorizontalBox();
      box.setAlignmentX(LEFT_ALIGNMENT);
      add(box);

      box.add(Box.createHorizontalStrut(INDENT));

      prefaceTeXButton = JMLRResources.createRadioButton(
        "newbook.frontmatter", "prefacetex", this, false);
      bg.add(prefaceTeXButton);
      box.add(prefaceTeXButton);
      prefaceTeXButton.setAlignmentX(LEFT_ALIGNMENT);

      prefaceFileField = new FileField(this, dialog.getFileChooser());
      prefaceFileField.setEnabled(false);
      box.add(prefaceFileField);

      add(Box.createVerticalGlue());

      forewordPanel = new JPanel(null);
      forewordPanel.setLayout(new BoxLayout(forewordPanel, 
         BoxLayout.PAGE_AXIS));
      forewordPanel.setAlignmentX(LEFT_ALIGNMENT);

      add(forewordPanel);

      label = 
        new JLabel(MakeJmlrBookGUI.getLabel("newbook.frontmatter.foreword"));
      label.setAlignmentX(LEFT_ALIGNMENT);

      forewordPanel.add(label);

      bg = new ButtonGroup();

      forewordNoButton = 
         JMLRResources.createRadioButton("newbook.frontmatter", "forewordno",
         this, true);
      forewordNoButton.setAlignmentX(LEFT_ALIGNMENT);
      bg.add(forewordNoButton);
      forewordPanel.add(forewordNoButton);

      forewordYesButton = 
         JMLRResources.createRadioButton("newbook.frontmatter", "forewordyes",
         this, false);
      forewordYesButton.setAlignmentX(LEFT_ALIGNMENT);
      bg.add(forewordYesButton);
      forewordPanel.add(forewordYesButton);

      box = Box.createHorizontalBox();
      box.setAlignmentX(LEFT_ALIGNMENT);
      forewordPanel.add(box);

      box.add(Box.createHorizontalStrut(INDENT));

      bg = new ButtonGroup();

      forewordNewButton = JMLRResources.createRadioButton(
         "newbook.frontmatter", "forewordnew", this, true);
      bg.add(forewordNewButton);
      box.add(forewordNewButton);
      forewordNewButton.setAlignmentX(LEFT_ALIGNMENT);
      forewordNewButton.setEnabled(false);

      box = Box.createHorizontalBox();
      box.setAlignmentX(LEFT_ALIGNMENT);
      forewordPanel.add(box);

      box.add(Box.createHorizontalStrut(INDENT));

      forewordTeXButton = JMLRResources.createRadioButton(
        "newbook.frontmatter", "forewordtex", this, false);
      bg.add(forewordTeXButton);
      box.add(forewordTeXButton);
      forewordTeXButton.setAlignmentX(LEFT_ALIGNMENT);
      forewordTeXButton.setEnabled(false);

      forewordFileField = new FileField(this, dialog.getFileChooser());
      forewordFileField.setEnabled(false);
      box.add(forewordFileField);

      add(Box.createVerticalGlue());
      add(Box.createVerticalStrut(100));
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("prefaceno"))
      {
         prefaceNewButton.setEnabled(false);
         prefaceTeXButton.setEnabled(false);
         prefaceFileField.setEnabled(false);
      }
      else if (action.equals("prefaceyes"))
      {
         prefaceNewButton.setEnabled(true);
         prefaceTeXButton.setEnabled(true);
         prefaceFileField.setEnabled(prefaceTeXButton.isSelected());
      }
      else if (action.equals("prefacenew"))
      {
         prefaceFileField.setEnabled(prefaceTeXButton.isSelected());
      }
      else if (action.equals("prefacetex"))
      {
         prefaceFileField.setEnabled(prefaceTeXButton.isSelected());
      }
      else if (action.equals("forewordno"))
      {
         forewordNewButton.setEnabled(false);
         forewordTeXButton.setEnabled(false);
         forewordFileField.setEnabled(false);
      }
      else if (action.equals("forewordyes"))
      {
         forewordNewButton.setEnabled(true);
         forewordTeXButton.setEnabled(true);
         forewordFileField.setEnabled(forewordTeXButton.isSelected());
      }
      else if (action.equals("forewordnew"))
      {
         forewordFileField.setEnabled(forewordTeXButton.isSelected());
      }
      else if (action.equals("forewordtex"))
      {
         forewordFileField.setEnabled(forewordTeXButton.isSelected());
      }
   }

   public void updatePanel()
   {
      forewordPanel.setVisible(dialog.isCiML());
   }

   public boolean canMoveForwards()
   {
      if (hasPreface() && prefaceTeXButton.isSelected())
      {
          File file = prefaceFileField.getFile();

          if (file == null)
          {
             JMLRResources.error(this, 
                MakeJmlrBookGUI.getLabel("newbook.frontmatter.missing_pref"));

             return false;
          }
          else if (!file.exists())
          {
             JMLRResources.error(this, 
                MakeJmlrBookGUI.getLabelWithValue("error.file_not_found",
                file.getAbsolutePath()));

             return false;
          }
      }

      if (hasForeword() && forewordTeXButton.isSelected())
      {
          File file = forewordFileField.getFile();

          if (file == null)
          {
             JMLRResources.error(this, 
                MakeJmlrBookGUI.getLabel("newbook.frontmatter.missing_fore"));

             return false;
          }
          else if (!file.exists())
          {
             JMLRResources.error(this, 
                MakeJmlrBookGUI.getLabelWithValue("error.file_not_found",
                file.getAbsolutePath()));

             return false;
          }
      }

      return true;
   }

   public boolean hasNextButton() {return true;}

   public boolean hasBackButton() {return true;}

   public boolean hasPreface()
   {
      return prefaceYesButton.isSelected();
   }

   public boolean hasForeword()
   {
      return forewordYesButton.isSelected();
   }

   public void setPrefaceFile(File file)
   {
      prefaceYesButton.setSelected(true);
      prefaceTeXButton.setSelected(true);
      prefaceFileField.setEnabled(true);
      prefaceFileField.setFile(file);
   }

   public File getPrefaceFile()
   {
      return prefaceTeXButton.isSelected() ? 
             prefaceFileField.getFile() : null ;
   }

   public File getForewordFile()
   {
      return forewordTeXButton.isSelected() ? 
             forewordFileField.getFile() : null ;
   }

   private JRadioButton prefaceTeXButton, prefaceNewButton,
      forewordTeXButton, forewordNewButton, prefaceNoButton,
      prefaceYesButton, forewordNoButton, forewordYesButton;

   private FileField prefaceFileField, forewordFileField;

   private JComponent forewordPanel;

   private NewBookDialog dialog;

   private final static int INDENT = 20;
}
