/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.io.IOException;
import java.io.File;
import java.util.List;
import java.util.Iterator;
import java.awt.Component;
import java.awt.datatransfer.*;

import javax.swing.TransferHandler;
import javax.swing.JComponent;

public class ImportPaperTransferHandler extends TransferHandler
{
   private ImportPaperTransferHandler()
   {
   }

   public ImportPaperTransferHandler(ArticlesPanel panel)
   {
      if (panel == null)
      {
         throw new NullPointerException();
      }

      this.articlesPanel = panel;
   }

   public Object clone()
   {
      return new ImportPaperTransferHandler(articlesPanel);
   }

   public int getSourceActions(JComponent c)
   {
      return MOVE;
   }

   protected Transferable createTransferable(JComponent c)
   {
      if (c instanceof ImportPaperPanel)
      {
         return new StringSelection(((ImportPaperPanel)c).getTransferData());
      }

      return null;
   }

   protected void exportDone(JComponent source, Transferable data, int action)
   {
      if (source instanceof ImportPaperPanel)
      {
         ((ImportPaperPanel)source).setSelected(false);
      }
   }

   public boolean canImport(TransferHandler.TransferSupport supp)
   {
      if (!(supp.getComponent() instanceof ImportPaperPanel))
      {
         return false;
      }

      return supp.isDataFlavorSupported(DataFlavor.stringFlavor)
          || supp.isDataFlavorSupported(DataFlavor.javaFileListFlavor);
   }

   public boolean importData(TransferHandler.TransferSupport supp)
   {
      if (!canImport(supp))
      {
         return false;
      }

      Component comp = supp.getComponent();

      ImportPaperPanel dropComp = (ImportPaperPanel)comp;

      Transferable trans = supp.getTransferable();

      try
      {
         List fileList = 
           (List)trans.getTransferData(DataFlavor.javaFileListFlavor);

         if (fileList != null)
         {
            for (Iterator it=fileList.iterator(); it.hasNext();)
            {
               File file = (File)it.next();

               if (!dropComp.hasFile())
               {
                  dropComp.setFile(file);
               }
               else
               {
                  ((ImportPapersPanel)dropComp.getParent()).addRow(file);
               }
            }

            return true;
         }
      }
      catch (UnsupportedFlavorException|IOException e)
      {
      }

      TransferHandler.DropLocation loc = supp.getDropLocation();

      String data = null;

      try
      {
         data = (String)trans.getTransferData(DataFlavor.stringFlavor);
      }
      catch (UnsupportedFlavorException|IOException e)
      {
         return false;
      }

      ImportPaperPanel dragComp = articlesPanel.getTransferComponent(data); 

      if (dragComp == null)
      {
          return false;
      }

      comp = dropComp.getParent();

      if (!(comp instanceof ImportPapersPanel))
      {
         return false;
      }

      ImportPapersPanel dropParent = (ImportPapersPanel)comp;

      if (dropParent == null)
      {
         return false;
      }

      comp = dragComp.getParent();

      if (!(comp instanceof ImportPapersPanel))
      {
         return false;
      }

      ImportPapersPanel dragParent = (ImportPapersPanel)comp;

      if (dragParent == null)
      {
         return false;
      }

      int dropIndex = dropParent.getImportPaperIndex(dropComp);
      int dragIndex = dragParent.getImportPaperIndex(dragComp);

      if (dropParent != dragParent)
      {
         dragParent.remove(dragIndex);
         dropParent.add(dragComp, dropIndex);

         if (dragParent.getComponentCount() == 0)
         {
            dragParent.addRow();
         }

         dragParent.updateButtons();
         dropParent.updateButtons();
         articlesPanel.revalidate();
         return true;
      }

      dragParent.remove(dragIndex);

      if (dropIndex < dropParent.getComponentCount())
      {
         dropParent.add(dragComp, dropIndex);
      }
      else
      {
         dropParent.add(dragComp);
      }

      dropParent.updateButtons();

      return true;
   }

   private ArticlesPanel articlesPanel;
}
