/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;

import com.dickimawbooks.texparserlib.*;
import com.dickimawbooks.texparserlib.bib.*;

import com.dickimawbooks.makejmlrbookgui.*;

public class PartListPanel extends JPanel
  implements ActionListener
{
   public PartListPanel(ArticlesPanel articlesPanel, int partNum,
     NewBookDialog dialog)
   {
      super(new BorderLayout());
      this.articlesPanel = articlesPanel;
      this.partNum = partNum;

      if (articlesPanel.getPart(partNum) != null)
      {
         throw new IllegalArgumentException("Duplicate part number "+partNum);
      }

      JPanel topPanel = new JPanel();
      add(topPanel, BorderLayout.NORTH);

      String partNumStr = String.format("%d", (partNum+1));

      partLabel = new JLabel(MakeJmlrBookGUI.getLabelWithValue(
         "newbook.articles.part", partNumStr));
      partLabel.setDisplayedMnemonic(
         partNumStr.charAt(partNumStr.length()-1));
      topPanel.add(partLabel);

      partField = new JTextField(20);
      partField.setText(MakeJmlrBookGUI.getLabel(
        "newbook.articles.partdefault"));
      partLabel.setLabelFor(partField);
      topPanel.add(partField);

      movePartUpButton = JMLRResources.createUpButton(this);
      topPanel.add(movePartUpButton);

      movePartDownButton = JMLRResources.createDownButton(this);
      topPanel.add(movePartDownButton);
      movePartDownButton.setEnabled(false);

      removePartButton = JMLRResources.createImageButton(
         "newbook.articles", "removepart", "general/Delete24.gif",
         this);

      topPanel.add(removePartButton);

      impPapersPanel = new ImportPapersPanel(dialog, articlesPanel, PART+partNum);

      add(impPapersPanel, BorderLayout.CENTER);

      partField.requestFocusInWindow();
   }

   public void setIndex(int index)
   {
      partNum = index;
      String partNumStr = ""+(partNum+1);

      partLabel.setText(MakeJmlrBookGUI.getLabelWithValue(
         "newbook.articles.part", partNumStr));
      partLabel.setDisplayedMnemonic(
         partNumStr.charAt(partNumStr.length()-1));
   }

   public void setRemoveEnabled(boolean enabled)
   {
      removePartButton.setEnabled(enabled);
   }

   public void setMovePartUpEnabled(boolean enabled)
   {
      movePartUpButton.setEnabled(enabled);
   }

   public void setMovePartDownEnabled(boolean enabled)
   {
      movePartDownButton.setEnabled(enabled);
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("removepart"))
      {
         if (JOptionPane.showConfirmDialog(this,
          MakeJmlrBookGUI.getLabelWithValue("newbook.articles.confirmrmpart",
            partField.getText()),
          MakeJmlrBookGUI.getLabel("message.confirm"),
          JOptionPane.YES_NO_OPTION)
          == JOptionPane.YES_OPTION)
         {
            articlesPanel.removePart(partNum);
         }
      }
   }

   public void updatePanel()
   {
      impPapersPanel.updatePanel();
   }

   public ImportPapersPanel getImportPapersPanel()
   {
      return impPapersPanel;
   }

   public String getPartTitle()
   {
      return partField.getText();
   }

   public void setPartTitle(String title)
   {
      partField.setText(title);
   }

   public int countPapers()
   {
      return impPapersPanel.countPapers();
   }

   public void addCimlImportFile(File base, BibEntry entry,
     TeXParser parser)
   {
      impPapersPanel.addCimlImportFile(base, entry, parser);
   }

   private ArticlesPanel articlesPanel;

   private JLabel partLabel;
   private JTextField partField;
   private JButton removePartButton, movePartUpButton, movePartDownButton;

   private ImportPapersPanel impPapersPanel;

   private int partNum;

   public final String PART="part";
}
