/*
    Copyright (C) 2013 Nicola L.C. Talbot
    www.dickimaw-books.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package com.dickimawbooks.makejmlrbookgui.newbook;

import java.util.Vector;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableColumn;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.dickimawbooks.texparserlib.bib.Contributor;

import com.dickimawbooks.makejmlrbookgui.*;

public class TitleListField extends JPanel 
  implements TitleField,ActionListener
{
   public TitleListField(String parentTag, String tag,
      final String columnLabel)
   {
      super(new FlowLayout(FlowLayout.LEADING, 0, 0));
      setAlignmentX(LEFT_ALIGNMENT);
      this.columnLabel = columnLabel;

      data = new Vector<String>();
      data.add(new String());

      TableModel model = new AbstractTableModel()
      {
         public int getColumnCount() {return data.size();}
         public int getRowCount() {return 1;}
         public boolean isCellEditable(int row, int col) {return true;}

         public Object getValueAt(int row, int col)
         {
            return data.get(col);
         }

         public void setValueAt(Object value, int row, int col)
         {
            data.set(col, (String)value);
         }

         public String getColumnName(int column)
         {
            return MakeJmlrBookGUI.getLabelWithValue(columnLabel, column+1);
         }
      };

      table = new JTable(model)
      {
         public void columnSelectionChanged(ListSelectionEvent e)
         {
            super.columnSelectionChanged(e);
            updateRemoveButton();
         }

         public String getToolTipText(MouseEvent e)
         {
            return MakeJmlrBookGUI.getLabel(columnLabel+".tooltip");
         }
      };

      table.setAlignmentX(LEFT_ALIGNMENT);
      table.setColumnSelectionAllowed(true);
      table.setRowSelectionAllowed(false);
      table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
      table.putClientProperty("terminateEditOnFocusLost", Boolean.TRUE);

      ListSelectionModel selectionModel = table.getSelectionModel();
      selectionModel.setSelectionMode(
         ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);

      JScrollPane sp = new JScrollPane(table);
      sp.setBorder(BorderFactory.createEmptyBorder());
      sp.setAlignmentX(LEFT_ALIGNMENT);

      Dimension dim = sp.getPreferredSize();
      dim.height = (int)table.getPreferredSize().getHeight() 
        + 3*table.getRowHeight();
      sp.setPreferredSize(dim);

      Dimension maxDim = sp.getMaximumSize();
      maxDim.height = dim.height;
      sp.setMaximumSize(maxDim);

      TableColumn column = table.getColumnModel().getColumn(0);
      column.setPreferredWidth(Math.max(column.getPreferredWidth(),
        dim.width/3));

      table.setRowHeight(table.getRowHeight()+4);

      add(sp);

      JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.LEADING));
      buttonPanel.setAlignmentX(LEFT_ALIGNMENT);

      JButton addButton = JMLRResources.createAddButton(this);
      buttonPanel.add(addButton);

      removeButton = JMLRResources.createRemoveButton(this);
      removeButton.setEnabled(false);
      buttonPanel.add(removeButton);

      String label = parentTag+"."+tag;

      labelComponent = Box.createVerticalBox();
      labelComponent.setAlignmentX(LEFT_ALIGNMENT);

      JLabel jLabel = new JLabel(MakeJmlrBookGUI.getLabel(label));
      jLabel.setAlignmentX(LEFT_ALIGNMENT);
      jLabel.setDisplayedMnemonic(MakeJmlrBookGUI.getMnemonic(label));

      labelComponent.add(jLabel);
      labelComponent.add(buttonPanel);
   }

   public void updateRemoveButton()
   {
      int colCount = table.getColumnCount();
      int selColCount = table.getSelectedColumnCount();

      removeButton.setEnabled(
        colCount > 1 && selColCount > 0 && selColCount < colCount);
   }

   public void actionPerformed(ActionEvent evt)
   {
      String action = evt.getActionCommand();

      if (action == null) return;

      if (action.equals("add"))
      {
         addRow();
      }
      else if (action.equals("remove"))
      {
         if (table.getSelectedColumnCount() > 0)
         {
            TableColumnModel colModel = table.getColumnModel();
            Vector<Integer> colWidths = new Vector<Integer>(data.size());

            for (int i = 0, n = data.size(); i < n; i++)
            {
               TableColumn column = colModel.getColumn(i);
               colWidths.add(column.getPreferredWidth());
            }

            int[] selection = table.getSelectedColumns();

            for (int i = selection.length-1; i >= 0; i--)
            {
               data.remove(selection[i]);
               colWidths.remove(selection[i]);
            }

            ((AbstractTableModel)table.getModel()).fireTableStructureChanged();

            for (int i = 0, n = colWidths.size(); i < n; i++)
            {
               TableColumn column = colModel.getColumn(i);
               column.setPreferredWidth(colWidths.get(i));
            }

            updateRemoveButton();
         }
      }
   }

   private void addRow()
   {
      addRows(null, 0);
   }

   private void addRows(java.util.List newEntries, int defWidth)
   {
      if (table.isEditing())
      {
         table.getCellEditor().stopCellEditing();
      }

      int numNewRows = (newEntries == null ? 1 : newEntries.size());

      int n = data.size();

      TableColumnModel colModel = table.getColumnModel();
      int[] colWidths = new int[n+numNewRows];

      int lastColWidth = defWidth;

      for (int i = 0; i < n; i++)
      {
         TableColumn column = colModel.getColumn(i);
         colWidths[i] = column.getPreferredWidth();
         lastColWidth = colWidths[i];
      }

      for (int i = 0; i < numNewRows; i++)
      {
         if (newEntries == null)
         {
            data.add(new String(""));
         }
         else
         {
            Object object = newEntries.get(i);

            if (object instanceof Contributor)
            {
               Contributor contributor = (Contributor)object;

               String forenames = contributor.getForenames();
               String von = contributor.getVonPart();
               String surname = contributor.getSurname();
               String suffix = contributor.getSuffix();

               StringBuffer buff = new StringBuffer();

               if (von != null)
               {
                  buff.append(von);
               }

               if (surname != null)
               {
                  if (buff.length() > 0) buff.append(" ");

                  buff.append(surname);
               }

               if (suffix != null)
               {
                  buff.append(", ");
                  buff.append(suffix);
               }

               String lastPart = buff.toString();

               if (lastPart.contains(" "))
               {
                  lastPart = "{"+lastPart+"}";
               }

               if (forenames == null)
               {
                  data.add(lastPart);
               }
               else
               {
                  data.add(forenames+" "+lastPart);
               }
            }
            else
            {
               data.add(object.toString());
            }
         }

         colWidths[i] = lastColWidth;
      }

      ((AbstractTableModel)table.getModel()).fireTableStructureChanged();

      for (int i = 0; lastColWidth != 0 && i < colWidths.length; i++)
      {
         TableColumn column = colModel.getColumn(i);
         column.setPreferredWidth(colWidths[i]);
      }

      updateRemoveButton();
   }

   public JComponent getComponent()
   {
      return this;
   }

   public JComponent getLabel()
   {
      return labelComponent;
   }

   public String getText()
   {
      return table.getColumnCount() == 0 ? "" : 
        table.getModel().getValueAt(0,0).toString();
   }

   public int getNumValues()
   {
      return table.getColumnCount();
   }

   public boolean isEmpty()
   {
      TableModel model = table.getModel();
      int n = model.getColumnCount();

      for (int i = 0; i < n; i++)
      {
         Object value = model.getValueAt(0, i);

         if (value != null && !value.toString().isEmpty())
         {
            return false;
         }
      }

      return true;
   }

   public void setFieldValue(Object object)
   {
      if (object instanceof java.util.List)
      {
         int prefWidth = 0;

         if (!data.isEmpty())
         {
            TableColumn column = table.getColumnModel().getColumn(0);
            prefWidth = column.getPreferredWidth();
         }

         data.clear();
         addRows((java.util.List)object, prefWidth);
      }
      else
      {
         if (data.isEmpty())
         {
            data.add(object.toString());
         }
         else
         {
            data.set(0, object.toString());
         }
      }

      ((AbstractTableModel)table.getModel()).fireTableStructureChanged();
      revalidate();
   }

   public Vector<String> getList()
   {
      return data;
   }

   public Vector<Contributor> getContributors()
   {
      Vector<Contributor> list = new Vector<Contributor>(data.size());

      for (String name : data)
      {
         name = name.trim();

         int idx = name.lastIndexOf(" ");

         if (idx == -1)
         {
            list.add(new Contributor("", name));
         }
         else
         {
            list.add(new Contributor(name.substring(0, idx),
               name.substring(idx+1)));
         }
      }

      return list;
   }

   public int getIntValue()
   {
      return 0;
   }

   private JTable table;

   private JButton removeButton;

   private Vector<String> data;

   private String columnLabel;
   private JComponent labelComponent;
}
